#define WIN32_LEAN_AND_MEAN
#define STRICT

#include "grdIconsWrapper.h"
#include "lsapi.h"

// blending raster
// the source image is inverted and the anded with the dest
#define INVERTSRCAND         0x00220326


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: grdIconsWrapper()                             * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HWND hTray                                            * */
/* *   the system tray window                                * */
/* *                                                         * */
/* *   HWND hParent                                          * */
/* *   the litestep window                                   * */
/* *                                                         * */
/* *   grdSysTraySettings *cfg                               * */
/* *   reference to the settings sub-class                   * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: initialize the iconsWrapper class              * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

grdIconsWrapper::grdIconsWrapper( HWND hTray, HWND hParent, grdSysTraySettings *cfg ) {

	// save the params
	this->hSysTray = hTray;
	this->hLiteStep = hParent;
	this->settings = cfg;

	// create a tooltip window
	this->hToolTip = CreateWindowEx( WS_EX_TOOLWINDOW | WS_EX_TOPMOST, TOOLTIPS_CLASS,
									 NULL,
									 TTS_ALWAYSTIP | WS_POPUP,
									 0,0,
									 0,0,
									 this->hSysTray,
									 NULL,
									 NULL,
									 NULL );

	this->uLastID = 1;

	this->pFirst = this->pLast = NULL;


}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: ~grdIconsWrapper()                            * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: destroy the iconsWrapper class                 * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

grdIconsWrapper::~grdIconsWrapper() {

	// destroy the tooltip window
	DestroyWindow( this->hToolTip );

	// destroy the list of icons
	this->delList();

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: BOOL delList()                                * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: empty the list of icons                        * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

BOOL grdIconsWrapper::delList() {

	PGRDSYSTRAYICON pTemporarySysTrayIcon, pTemporarySysTrayIcon2;

	// 	delete the items in the wrapper
	if ( this->pFirst ) {

		// start iterating the history items
		pTemporarySysTrayIcon = this->pFirst;

		do {

			// save the one to be deleted and move to the next
			pTemporarySysTrayIcon2 = pTemporarySysTrayIcon;
			pTemporarySysTrayIcon  = pTemporarySysTrayIcon->pNext;

			// delete this history item
			
			delete pTemporarySysTrayIcon2;

		} while ( pTemporarySysTrayIcon != NULL );

	}
	return TRUE;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: PGRDSYSTRAYICON add()                         * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   PNOTIFYICONDATA pnid                                  * */
/* *   data structure with information about the icon        * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: add icon to the list                           * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

PGRDSYSTRAYICON grdIconsWrapper::add( PNOTIFYICONDATA pnid ) {

	PGRDSYSTRAYICON pTemporarySysTrayIcon;
	TOOLINFO ti;
	
	if( !IsWindow( pnid->hWnd ) )
		return NULL;

	// check if the icon already exists
	pTemporarySysTrayIcon = this->find( pnid->hWnd, pnid->uID );

	// if found, modify instead of add
	if ( pTemporarySysTrayIcon != NULL ) 
		return this->modify( pnid );

	// if not found (which is the way it should go)
	pTemporarySysTrayIcon = new GRDSYSTRAYICON;

	pTemporarySysTrayIcon->hWnd				= pnid->hWnd;
	pTemporarySysTrayIcon->uID				= pnid->uID;
	pTemporarySysTrayIcon->uFlags			= pnid->uFlags;

	pTemporarySysTrayIcon->uCallbackMessage = ( pTemporarySysTrayIcon->uFlags & NIF_MESSAGE ) ? pnid->uCallbackMessage : NULL;

	//pTemporarySysTrayIcon->hOriginalIcon	= pnid->hIcon;
	pTemporarySysTrayIcon->hIcon			= ( pTemporarySysTrayIcon->uFlags & NIF_ICON ) ? CopyIcon( pnid->hIcon ) : NULL;

	if ( pTemporarySysTrayIcon->uFlags & NIF_TIP ) {

		pTemporarySysTrayIcon->szTip = new char[strlen(pnid->szTip)+1];
		pTemporarySysTrayIcon->szTip[0] = '\0';

		strcpy( pTemporarySysTrayIcon->szTip, pnid->szTip );
	
	} else {

		pTemporarySysTrayIcon->szTip = NULL;

	}

	pTemporarySysTrayIcon->uToolTip = this->uLastID++;

	// add to last position in the list
	if ( !this->pFirst ) {

		this->pFirst = pTemporarySysTrayIcon;
		pTemporarySysTrayIcon->pPrev = NULL;

	} 
		
	if ( this->pLast ) {

		pTemporarySysTrayIcon->pPrev = this->pLast;
		pTemporarySysTrayIcon->pPrev->pNext = pTemporarySysTrayIcon;

	}

	this->pLast = pTemporarySysTrayIcon;

	pTemporarySysTrayIcon->pNext = NULL;
	
	/* **************************** */
	/* *   ADJUST THE RECTANGLE   * */
	/* **************************** */

	if ( pTemporarySysTrayIcon->pPrev ) {

		if ( !(this->settings->uDirection & TD_VERTICAL) ) {

			// if there is room for one more icon
			if ( pTemporarySysTrayIcon->pPrev->rc.left != this->settings->nLastX ) {

				pTemporarySysTrayIcon->rc.left = pTemporarySysTrayIcon->pPrev->rc.left + this->settings->nDeltaX;
				pTemporarySysTrayIcon->rc.top = pTemporarySysTrayIcon->pPrev->rc.top;
			
			} else {

				pTemporarySysTrayIcon->rc.left = this->settings->nFirstX;
				pTemporarySysTrayIcon->rc.top = pTemporarySysTrayIcon->pPrev->rc.top + this->settings->nDeltaY;

			}

		} else {

			// if there is room for one more icon
			if ( pTemporarySysTrayIcon->pPrev->rc.top != this->settings->nLastY ) {

				pTemporarySysTrayIcon->rc.top = pTemporarySysTrayIcon->pPrev->rc.top + this->settings->nDeltaY;
				pTemporarySysTrayIcon->rc.left = pTemporarySysTrayIcon->pPrev->rc.left;
			
			} else {

				pTemporarySysTrayIcon->rc.top = this->settings->nFirstY;
				pTemporarySysTrayIcon->rc.left = pTemporarySysTrayIcon->pPrev->rc.left + this->settings->nDeltaX;

			}

		}

	} else {

		pTemporarySysTrayIcon->rc.left = this->settings->nFirstX;
		pTemporarySysTrayIcon->rc.top = this->settings->nFirstY;

	}


	pTemporarySysTrayIcon->rc.right = pTemporarySysTrayIcon->rc.left + this->settings->uIconSize;
	pTemporarySysTrayIcon->rc.bottom = pTemporarySysTrayIcon->rc.top + this->settings->uIconSize;

	/* **************************** */
	/* *    END OF ADJUSTATION    * */
	/* **************************** */
	
	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = 0;
	ti.hwnd = this->hSysTray;
	ti.uId = pTemporarySysTrayIcon->uToolTip;
	ti.rect = pTemporarySysTrayIcon->rc;
	ti.hinst = NULL;
	ti.lpszText = ( pTemporarySysTrayIcon->uFlags & NIF_TIP ) ? pTemporarySysTrayIcon->szTip : NULL;
	
	SendMessage( this->hToolTip, TTM_ADDTOOL, 0, (LPARAM) &ti );

	// remove "unwanted" icons
	this->cleanup();

	// return the newly created icon
	return pTemporarySysTrayIcon;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: PGRDSYSTRAYICON del()                         * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   PNOTIFYICONDATA pnid                                  * */
/* *   data structure with information about the icon        * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: remove icon from the list                      * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

BOOL grdIconsWrapper::del( PNOTIFYICONDATA pnid ) {

	PGRDSYSTRAYICON pSysTrayIcon;
	
	pSysTrayIcon = this->find( pnid->hWnd, pnid->uID );
	
	return this->delInside( pSysTrayIcon );

}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: BOOL delInside()                              * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   PGRDSYSTRAYICON pSysTrayIcon                          * */
/* *   data structure with information about the icon        * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: remove icon from the list                      * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

BOOL grdIconsWrapper::delInside( PGRDSYSTRAYICON pSysTrayIcon ) {
	
	PGRDSYSTRAYICON pIteratorSysTrayIcon;
	TOOLINFO ti;

	if( !pSysTrayIcon )
		return FALSE;

	if( pSysTrayIcon->uFlags & NIF_ICON )
		DestroyIcon( pSysTrayIcon->hIcon );
	
	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = this->hSysTray;
	ti.uId = pSysTrayIcon->uToolTip;
	
	SendMessage( this->hToolTip, TTM_DELTOOL, 0, (LPARAM) &ti );

	// delete the tip, if there is any
	if ( pSysTrayIcon->szTip )
		delete[] pSysTrayIcon->szTip;

	// change all the rectangles and update the tooltips
	pIteratorSysTrayIcon = this->pLast;

	while ( pIteratorSysTrayIcon != pSysTrayIcon ) {

		pIteratorSysTrayIcon->rc = pIteratorSysTrayIcon->pPrev->rc;

		if( pIteratorSysTrayIcon->uFlags & NIF_TIP ) {

			ti.hwnd = this->hSysTray;
			ti.uId = pIteratorSysTrayIcon->uToolTip;
			ti.rect = pIteratorSysTrayIcon->rc;
			
			SendMessage( hToolTip, TTM_NEWTOOLRECT, 0, (LPARAM) &ti );

		}

		pIteratorSysTrayIcon = pIteratorSysTrayIcon->pPrev;

	}

	// if it's first or last, move those pointers
	if ( this->pFirst == pSysTrayIcon )
		this->pFirst = pSysTrayIcon->pNext;
	else
		pSysTrayIcon->pPrev->pNext = pSysTrayIcon->pNext;


	if ( this->pLast == pSysTrayIcon )
		this->pLast = pSysTrayIcon->pPrev;
	else
		pSysTrayIcon->pNext->pPrev = pSysTrayIcon->pPrev;

	delete pSysTrayIcon;

	return TRUE;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: PGRDSYSTRAYICON modify()                      * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   PNOTIFYICONDATA pnid                                  * */
/* *   data structure with information about the icon        * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: modify icon in the list                        * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

PGRDSYSTRAYICON grdIconsWrapper::modify( PNOTIFYICONDATA pnid ) {

	PGRDSYSTRAYICON pTemporarySysTrayIcon;
	TOOLINFO ti;
	
	// check if the icon already exists
	pTemporarySysTrayIcon = this->find( pnid->hWnd, pnid->uID );

	// if not found, add instead of modify
	if ( pTemporarySysTrayIcon == NULL ) 
		return this->add( pnid );


	if( pnid->uFlags & NIF_MESSAGE ) {

		pTemporarySysTrayIcon->uFlags |= NIF_MESSAGE;
		pTemporarySysTrayIcon->uCallbackMessage = pnid->uCallbackMessage;

	}
	
	if( pnid->uFlags & NIF_TIP ) {

		// remove remains of old tip
		pTemporarySysTrayIcon->uFlags ^= NIF_TIP;

		if ( pTemporarySysTrayIcon->szTip )
			delete[] pTemporarySysTrayIcon->szTip;

		// new tip to add
		if ( strlen( pnid->szTip ) ) {

			pTemporarySysTrayIcon->szTip = new char[ strlen( pnid->szTip ) + 1 ];
			pTemporarySysTrayIcon->szTip[0] = '\0';

			strcpy( pTemporarySysTrayIcon->szTip, pnid->szTip );

			pTemporarySysTrayIcon->uFlags |= NIF_TIP;

		} else {

			pTemporarySysTrayIcon->szTip = NULL;

		}
		
		ti.cbSize = sizeof(TOOLINFO);
		ti.hwnd = this->hSysTray;
		ti.uId = pTemporarySysTrayIcon->uToolTip;
		ti.hinst = NULL;
		ti.lpszText = pTemporarySysTrayIcon->szTip;
		
		SendMessage( this->hToolTip, TTM_UPDATETIPTEXT, 0, (LPARAM) &ti );

	}
	
	if( pnid->uFlags & NIF_ICON ) {

		// delete old icon (if there is one)
		if ( pTemporarySysTrayIcon->hIcon != NULL )
			DestroyIcon( pTemporarySysTrayIcon->hIcon );
		
		pTemporarySysTrayIcon->uFlags |= NIF_ICON;

		pTemporarySysTrayIcon->hIcon = CopyIcon( pnid->hIcon );
		
		//InvalidateRect( hWnd, NULL, TRUE ); //&psti->rc, FALSE );

	}

	// remove "unwanted" icons
	this->cleanup();

	// return the "refreshed" icon
	return pTemporarySysTrayIcon;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: PGRDSYSTRAYICON find()                        * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HWND hWnd                                             * */
/* *   icon's owner window                                   * */
/* *                                                         * */
/* *   UINT uID                                              * */
/* *   an ID identifying the icon                            * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: find icon in the list based on owner and ID    * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

PGRDSYSTRAYICON grdIconsWrapper::find( HWND hWnd, UINT uID ) {

	PGRDSYSTRAYICON pTemporarySysTrayIcon = this->pFirst;
	
	while( pTemporarySysTrayIcon ) {

		if( ( pTemporarySysTrayIcon->hWnd == hWnd ) && ( pTemporarySysTrayIcon->uID == uID ) )
			return pTemporarySysTrayIcon;
		
		pTemporarySysTrayIcon = pTemporarySysTrayIcon->pNext;

	}
	
    return NULL;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: PGRDSYSTRAYICON find()                        * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HWND hWnd                                             * */
/* *   icon's owner window                                   * */
/* *                                                         * */
/* *   UINT uID                                              * */
/* *   an ID identifying the icon                            * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: find icon in the list based on owner and ID    * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

void grdIconsWrapper::cleanup() {

	PGRDSYSTRAYICON pTemporarySysTrayIcon = this->pFirst;
	
	while( pTemporarySysTrayIcon ) {

		if( !IsWindow( pTemporarySysTrayIcon->hWnd )  )
			this->delInside( pTemporarySysTrayIcon );
		
		pTemporarySysTrayIcon = pTemporarySysTrayIcon->pNext;

	}
	
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: PGRDSYSTRAYICON findPt()                      * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   POINT pt                                              * */
/* *   the point where there should be an icon beneath?      * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: find icon in the list based on a point         * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

PGRDSYSTRAYICON grdIconsWrapper::findPt( POINT pt ) {

	PGRDSYSTRAYICON pTemporarySysTrayIcon = this->pFirst;
	
	while( pTemporarySysTrayIcon ) {

		if( PtInRect( &pTemporarySysTrayIcon->rc, pt ) )

			// return NULL if it's parent is dead
			return ( IsWindow( pTemporarySysTrayIcon->hWnd ) ) ? pTemporarySysTrayIcon : NULL;
		
		pTemporarySysTrayIcon = pTemporarySysTrayIcon->pNext;

	}
	
    return NULL;

}



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: void iconsOut()                               * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HDC hdcDest                                           * */
/* *   the destination dc                                    * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: print out the icons on a specific dc           * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

void grdIconsWrapper::iconsOut( HDC hdcDest ) {

	PGRDSYSTRAYICON pTemporarySysTrayIcon = this->pFirst;

	while( pTemporarySysTrayIcon ) {

        if( pTemporarySysTrayIcon->uFlags & NIF_ICON ) {

			if( this->settings->uEffectFlags & FX_ANY ) {

				HDC hdcMask, hdcMem;
				HBITMAP hbmMaskOld;

				ICONINFO iconInfo;
					
				// load the icons HBITMAPs (mask & color bitmap)
				GetIconInfo( pTemporarySysTrayIcon->hIcon, &iconInfo );

				// load the MASK into one dc
				hdcMask    = CreateCompatibleDC( hdcDest );
				hbmMaskOld = (HBITMAP) SelectObject( hdcMask, iconInfo.hbmMask );

				// create a dc consist of the new bitmap
				hdcMem = CreateCompatibleDC( hdcDest );

				if ( hdcMem ) {

					BITMAP bm;					// Get bitmap size
					HBITMAP hbm32;				// the new 32 bit bitmap
					BITMAPINFOHEADER bmInfo;	// bitmapinfoheader for above
					VOID *pbits32;

					// get bitmap size
					GetObject( iconInfo.hbmColor, sizeof(bm), &bm ); 

					// set everything to 0
					memset( &bmInfo, 0, sizeof(BITMAPINFOHEADER) );

					// Create a 32 bits depth bitmap
					bmInfo.biSize = sizeof(BITMAPINFOHEADER);
					bmInfo.biWidth = bm.bmWidth;
					bmInfo.biHeight = bm.bmHeight;
					bmInfo.biPlanes = 1;
					bmInfo.biBitCount = 32;
					bmInfo.biCompression = BI_RGB;

					hbm32 = CreateDIBSection( hdcMem,
											  (BITMAPINFO*) &bmInfo,
											  DIB_RGB_COLORS,
											  &pbits32,
											  NULL,
											  0);

					if ( hbm32 ) {

						// select the new bitmap into memory
						HBITMAP hbmOld32 = (HBITMAP) SelectObject( hdcMem, hbm32 );

						// create a new dc for copying
						HDC hdcSrc = CreateCompatibleDC( hdcDest );

						if ( hdcSrc ) {

							BITMAP bm32;
							HBITMAP hbmOldSrc;
							BYTE *p32;
							int y, x;

							// get the bits from hbm32
							GetObject( hbm32, sizeof(bm32), &bm32 );

							// Get how many bytes per row we have for the bitmap bits (rounded up to 32 bits)
							while ( bm32.bmWidthBytes % 4 )
								bm32.bmWidthBytes++;

							// copy the original bitmap into it's dc
							hbmOldSrc = (HBITMAP) SelectObject( hdcSrc, iconInfo.hbmColor );
							// copy the original bitmaps contents into the new one
							BitBlt(hdcMem, 0, 0, bm.bmWidth, bm.bmHeight, hdcSrc, 0, 0, SRCCOPY);

							// scan each bitmap row from bottom to top (the bitmap is inverted vertically)
							p32 = (BYTE*) bm32.bmBits + (bm32.bmHeight - 1) * bm32.bmWidthBytes;

							for (y = 0; y < bm.bmHeight; y++) {

								// go through every pixel from left to right
								for (x = 0; x < bm.bmWidth; x++) {

									// *p gets the color value of the pixel,
									// which can be freely modified
									DWORD *p = (DWORD*) p32 + x;
									BYTE   r = GetRValue( *p );
									BYTE   g = GetGValue( *p );
									BYTE   b = GetBValue( *p );

									// saturnation effect
									if ( this->settings->uEffectFlags & FX_SATURNATION ) {

										float fSaturnation = (float)this->settings->uSaturnation / (float)255;

										BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

										r = (BYTE)( r * fSaturnation + gray * (1 - fSaturnation) );
										g = (BYTE)( g * fSaturnation + gray * (1 - fSaturnation) );
										b = (BYTE)( b * fSaturnation + gray * (1 - fSaturnation) );

									}

									// icon hue effect
									if ( this->settings->uEffectFlags & FX_ICONHUE ) {

										float fHueIntensity = (float)this->settings->uHueIntensity / (float)255;

										// the B & R values of the hue is swapped here, can somebody tell me why it only works that way?
										r = (BYTE)( r * (1 - fHueIntensity) + GetBValue(this->settings->clrHue) * fHueIntensity );
										g = (BYTE)( g * (1 - fHueIntensity) + GetGValue(this->settings->clrHue) * fHueIntensity );
										b = (BYTE)( b * (1 - fHueIntensity) + GetRValue(this->settings->clrHue) * fHueIntensity );

									}

									*p = RGB( r, g, b );

								}

								// Go to next row (remember, the bitmap is inverted vertically)
								p32 -= bm32.bmWidthBytes;
							}

							// Clean up
							SelectObject( hdcSrc, hbmOldSrc );
							DeleteDC( hdcSrc );

							// remove unwanted information from the modified			
							BitBlt( hdcMem,0,0,bm.bmWidth,bm.bmHeight,hdcMask,0,0,INVERTSRCAND );

							// do an "transparency blit onto the destionation buffer, using mask as mask and icon as source
							StretchBlt( hdcDest, pTemporarySysTrayIcon->rc.left, pTemporarySysTrayIcon->rc.top, this->settings->uIconSize, this->settings->uIconSize, hdcMask, 0, 0, bm.bmWidth, bm.bmHeight, SRCAND );
							// Combine the foreground with the background
							StretchBlt( hdcDest, pTemporarySysTrayIcon->rc.left, pTemporarySysTrayIcon->rc.top, this->settings->uIconSize, this->settings->uIconSize, hdcMem , 0, 0, bm.bmWidth, bm.bmHeight, SRCPAINT );

						}
						SelectObject( hdcMem, hbmOld32 );
						DeleteObject( hbm32 );

					}
					DeleteDC( hdcMem );

				}

				SelectObject( hdcMask, hbmMaskOld );
				DeleteDC( hdcMask );

				DeleteObject( iconInfo.hbmMask  );
				DeleteObject( iconInfo.hbmColor );


			} else {

				DrawIconEx( hdcDest, pTemporarySysTrayIcon->rc.left, pTemporarySysTrayIcon->rc.top, pTemporarySysTrayIcon->hIcon, this->settings->uIconSize, this->settings->uIconSize, 0, NULL, DI_NORMAL );

			}

		}

		pTemporarySysTrayIcon = pTemporarySysTrayIcon->pNext;

	}

}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: int save()                                    * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   void *dst                                             * */
/* *   the destination buffer                                * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: save icon structures into the buffer           * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

int grdIconsWrapper::save(void *dst) {

	NOTIFYICONDATA *trayArray = (NOTIFYICONDATA*)dst;
	PGRDSYSTRAYICON pTemporarySysTrayIcon = this->pFirst;

	int i = 0;

	while ( pTemporarySysTrayIcon ) {

		// set the saving data based on the list
		trayArray[i].cbSize = sizeof(NOTIFYICONDATA);
		trayArray[i].hIcon  = CopyIcon( pTemporarySysTrayIcon->hIcon );
		trayArray[i].hWnd   = pTemporarySysTrayIcon->hWnd;
		strcpy( trayArray[i].szTip, pTemporarySysTrayIcon->szTip );
		trayArray[i].uCallbackMessage = pTemporarySysTrayIcon->uCallbackMessage;
		trayArray[i].uFlags = pTemporarySysTrayIcon->uFlags;
		trayArray[i].uID = pTemporarySysTrayIcon->uID;

		i++;
		pTemporarySysTrayIcon = pTemporarySysTrayIcon->pNext;

	}

	trayArray[i].hWnd = 0;

	return i;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: int load()                                    * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   void *src                                             * */
/* *   the source buffer                                     * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: load icon structures from the buffer           * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

int grdIconsWrapper::load(void *src) {

	NOTIFYICONDATA *trayArray = (NOTIFYICONDATA*)src;

	int i = 0;

	if ( src != NULL ) {

		while ( trayArray[i].hWnd != 0 ) {

			// add all icons from the buffer
			SendMessage( this->hLiteStep, LM_SYSTRAY, NIM_ADD, (LPARAM)&trayArray[i] );

			DeleteObject( trayArray[i].hIcon );
			i++;

		}

	}

	return i;

}
