#define WIN32_LEAN_AND_MEAN
#define STRICT
	
#include <windows.h>
#include <shellapi.h>
#include <stdio.h>
#include "lsapi.h"
#include "grdShellTray.h"

// don't know the purpose of this one...
#define MAGICDWORD         0x49474541

// pass message structure
typedef struct SHELLTRAYDATA {
	DWORD dwUnknown;
	DWORD dwMessage;
	NOTIFYICONDATA nid;
} SHELLTRAYDATA, *PSHELLTRAYDATA, FAR *LPSHELLTRAYDATA;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: grdShellTray()                                * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:
/* *                                                         * */
/* *   HWND hTray                                            * */
/* *   the "parent" tray window                              * */
/* *                                                         * */
/* *   HINSTANCE dll                                         * */
/* *   the current instance                                  * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: initialize the shellTray class                 * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

grdShellTray::grdShellTray ( HWND lswnd, HINSTANCE lsinstance ) {

	// temporary variables
	WNDCLASSEX wc;

	/* *************************************** */
	/* *     STORE THE INFORMATION PASSED    * */
	/* *************************************** */

	this->hInstance = lsinstance;
	this->hLiteStep = lswnd;

	
	/* *************************************** */
	/* *     REGISTER THE SHELLTRAY CLASS    * */
	/* *************************************** */
	
   	memset(&wc,0,sizeof(wc));

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_DBLCLKS ;
	wc.lpfnWndProc = (WNDPROC) grdShellTray::WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 4;
	wc.hInstance = this->hInstance;
	wc.lpszClassName = WC_SHELLTRAY;
		
   	if (!RegisterClassEx(&wc))
 		MessageBox( this->hLiteStep, "Error registering window class", WC_SHELLTRAY, MB_OK );
		

	/* *************************************** */
	/* *     CREATE THE SHELLTRAY WINDOW     * */
	/* *************************************** */
	
	this->hShellTray = ::CreateWindowEx( WS_EX_TOPMOST | WS_EX_TOOLWINDOW,
							             WC_SHELLTRAY,
									     "grdSysTrayManager",
									     WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
									     0,0,
									     10,10,
									     this->hLiteStep,
									     NULL,
									     this->hInstance,
									     NULL );

	if ( !this->hShellTray )
		MessageBox( this->hLiteStep,"Error creating window", WC_SHELLTRAY, MB_OK);


	/* *************************************** */
	/* *      ADJUST THE SYSTRAY WINDOW      * */
	/* *************************************** */
	
	// set the magicDWord
	SetWindowLong( this->hShellTray, GWL_USERDATA, (LONG)MAGICDWORD );

	// save the class handle as a window long
	SetWindowLong( this->hShellTray, 0, (LONG)this );
	
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: ~grdShellTray()                               * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: destroy the shellTray class                    * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

grdShellTray::~grdShellTray ( ) {

	// the window could be deleted
	DestroyWindow( this->hShellTray );

	// the class could be unregistered
	UnregisterClass( WC_SHELLTRAY, this->hInstance );

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: LRESULT WndProc()                             * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HWND hWnd                                             * */
/* *   a handle to a window which this function handles      * */
/* *                                                         * */
/* *   UINT nMessage                                         * */
/* *   the message sent to the window                        * */
/* *                                                         * */
/* *   WPARAM wParam                                         * */
/* *   the WPARAM of this message                            * */
/* *                                                         * */
/* *   LPARAM lParam                                         * */
/* *   the LPARAM of this message                            * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: forward the window's messages                  * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

LRESULT CALLBACK grdShellTray::WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {	

	grdShellTray *theShellTray = (grdShellTray*)::GetWindowLong(hwnd, 0);
	
	if (theShellTray)
		return theShellTray->WindowProc(hwnd, message, wParam, lParam);

	else 
        return DefWindowProc(hwnd, message, wParam, lParam);

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: LRESULT WindowProc()                          * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HWND hWnd                                             * */
/* *   a handle to a window which this function handles      * */
/* *                                                         * */
/* *   UINT nMessage                                         * */
/* *   the message sent to the window                        * */
/* *                                                         * */
/* *   WPARAM wParam                                         * */
/* *   the WPARAM of this message                            * */
/* *                                                         * */
/* *   LPARAM lParam                                         * */
/* *   the LPARAM of this message                            * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: handle the window's messages                   * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

LRESULT grdShellTray::WindowProc( HWND hWnd, UINT nMessage, WPARAM wParam, LPARAM lParam ) {

	switch( nMessage )
	{

		case WM_COPYDATA:
		{
			NOTIFYICONDATA nid;
			PCOPYDATASTRUCT pcds;
			PSHELLTRAYDATA pstd;
			
			pcds = (PCOPYDATASTRUCT) lParam;
			
			if( pcds->dwData != 1 )
				return FALSE;
			
			pstd = (PSHELLTRAYDATA) pcds->lpData;

			nid.cbSize = sizeof(NOTIFYICONDATA);
			nid.hWnd = pstd->nid.hWnd;
			nid.uID = pstd->nid.uID;
			nid.uFlags = pstd->nid.uFlags;
			nid.uCallbackMessage = pstd->nid.uCallbackMessage;
			nid.hIcon = pstd->nid.hIcon;
			nid.szTip[0] = 0;
	
			if( nid.uFlags & NIF_TIP )
				lstrcpyn( nid.szTip, pstd->nid.szTip, 64 );
	
			return (BOOL) ::SendMessage( this->hLiteStep, LM_SYSTRAY, pstd->dwMessage, (LPARAM) &nid );
			
		}
		
	}
	
	return DefWindowProc( hWnd, nMessage, wParam, lParam );
}

