#define WIN32_LEAN_AND_MEAN
#define STRICT
	
#include <windows.h>
#include <stdlib.h>
#include "lsapi.h"
#include "grdSysTray.h"


// don't know the purpose of this one...
#define MAGICDWORD         0x49474541


// register messages from litestep
UINT nMessages[] = {
//  LM_GETREVID,
	LM_RESTORESYSTRAY, // removed due to testing
	LM_SAVESYSTRAY,    // removed due to testing
//	LM_SENDSYSTRAY,    // removed due to testing
	LM_SYSTRAY,
	0
};

void grdTrayHide( HWND hCaller, LPCTSTR pszArgs );
void grdTrayShow( HWND hCaller, LPCTSTR pszArgs );
void grdTrayToggle( HWND hCaller, LPCTSTR pszArgs );
void grdTrayMove( HWND hCaller, LPCTSTR pszArgs );

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: grdSysTray()                                  * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   LPCTSTR szPath                                        * */
/* *   path to the litestep directory                        * */
/* *                                                         * */
/* *   HINSTANCE dll                                         * */
/* *   the current instance                                  * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: initialize the sysTray class                   * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

grdSysTray::grdSysTray( HWND lswnd, HINSTANCE lsinstance, LPCTSTR lspath ) {

	// temporary variables
	WNDCLASSEX wc;
	
	/* *************************************** */
	/* *     STORE THE INFORMATION PASSED    * */
	/* *************************************** */

	this->hLiteStep = lswnd;
    this->hInstance = lsinstance;

	this->szLSPath = new char[ strlen( lspath ) + 1 ];
	strcpy( this->szLSPath, lspath );


	/* *************************************** */
	/* *            LOAD SETTINGS            * */
	/* *************************************** */

	this->settings = new grdSysTraySettings();


	/* *************************************** */
	/* *       GET SOME WINDOW HANDLES       * */
	/* *************************************** */

	this->hDesktop = FindWindow(WC_SHELLDESKTOP, NULL);
	if ( !this->hDesktop )
		this->hDesktop = GetDesktopWindow();


	/* *************************************** */
	/* *       INITIATE SOME VARIABLES       * */
	/* *************************************** */
	
	this->hSysTray  = NULL;
	this->icons     = NULL; 
	this->shellTray = NULL;


	/* *************************************** */
	/* *      REGISTER THE SYSTRAY CLASS     * */
	/* *************************************** */

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_DBLCLKS | CS_GLOBALCLASS;
	wc.lpfnWndProc = (WNDPROC) grdSysTray::WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 4;
	wc.hInstance = this->hInstance;
	wc.hCursor = LoadCursor( NULL, IDC_ARROW );
	wc.hIcon = NULL;
	wc.hIconSm = NULL;
	wc.hbrBackground = NULL;
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WC_SYSTRAY;
	
	if (!RegisterClassEx( &wc ) ) 
        MessageBox(0,"couldn't register the window class","ERROR!",MB_OK);
	

	/* *************************************** */
	/* *      CREATE THE SYSTRAY WINDOW      * */
	/* *************************************** */

	this->hSysTray = CreateWindowEx( WS_EX_TOOLWINDOW | ( this->settings->bAlwaysOnTop ) ? WS_EX_TOPMOST : NULL,
						             WC_SYSTRAY,
									 NULL,
									 WS_POPUP,
									 this->settings->nTrayX, this->settings->nTrayY,
									 this->settings->nTrayWidth, this->settings->nTrayHeight,
									 this->hLiteStep,
									 NULL,
									 this->hInstance,
									 NULL );

	if ( this->hSysTray == NULL )
		return;


	/* *************************************** */
	/* *      ADJUST THE SYSTRAY WINDOW      * */
	/* *************************************** */
	
	// set the magicDWord
	SetWindowLong( this->hSysTray, GWL_USERDATA, (LONG)MAGICDWORD );

	// save the class handle as a window long
	SetWindowLong( this->hSysTray, 0, (LONG)this );

	// set always on top
	if( this->settings->bAlwaysOnTop )
		SetWindowPos( this->hSysTray, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE );

	// set the parent window to the desktop window
	//
	// when it's always on top this is avoided
	// don't know about this, but I think the topmost
	// doesn't work otherwise
	//
	// please correct me if I'm wrong
	if( !this->settings->bAlwaysOnTop ) 
		SetParent( this->hSysTray, this->hDesktop );
	

	/* *************************************** */
	/* *       LAUNCH THE ICON-WRAPPER       * */
	/* *************************************** */
	
	this->icons = new grdIconsWrapper( this->hSysTray, this->hLiteStep, this->settings );
	
	
	/* *************************************** */
	/* *     SHOW AND UPDATE THE SYSTRAY     * */
	/* *************************************** */	

	ShowWindow( this->hSysTray, ( this->settings->bVisible && !this->settings->bHideIfEmpty ) ? SW_SHOWNOACTIVATE : SW_HIDE );
	InvalidateRect( this->hSysTray, NULL, TRUE );


	/* *************************************** */
	/* *  ACTIVATE IT AS A LITESTEP SYSTRAY  * */
	/* *************************************** */

	SendMessage( this->hLiteStep, LM_REGISTERMESSAGE, (WPARAM) this->hSysTray, (LPARAM) nMessages );


	/* *************************************** */
	/* *          LOAD BANG COMMANDS         * */
	/* *************************************** */

	AddBangCommand( "!grdTrayHide",   grdTrayHide   );
	AddBangCommand( "!grdTrayShow",   grdTrayShow   );
	AddBangCommand( "!grdTrayToggle", grdTrayToggle );
	AddBangCommand( "!grdTrayMove",   grdTrayMove   );

    
	/* *************************************** */
	/* *   CREATE A SHELLTRAY IF NECESSARY   * */
	/* *************************************** */
	
	if( !FindWindow( WC_SHELLTRAY, NULL ) )
		this->shellTray = new grdShellTray( this->hLiteStep, this->hInstance );

	return;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: ~grdSysTray()                                 * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: destroy the sysTray class                      * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

grdSysTray::~grdSysTray( ) {

	/* *************************************** */
	/* *         UNLOAD BANG COMMANDS        * */
	/* *************************************** */
	
	RemoveBangCommand( "!grdTrayHide"   );
	RemoveBangCommand( "!grdTrayShow"   );
	RemoveBangCommand( "!grdTrayToggle" );
	RemoveBangCommand( "!grdTrayMove"   );


	/* *************************************** */
	/* * DEACTIVATE IT AS A LITESTEP SYSTRAY * */
	/* *************************************** */

	SendMessage( this->hLiteStep, LM_UNREGISTERMESSAGE, (WPARAM) this->hSysTray, (LPARAM) nMessages );


	/* *************************************** */
	/* *     DESTROY THE SYSTRAY WINDOW      * */
	/* *************************************** */
	
	DestroyWindow( this->hSysTray );
	this->hSysTray = NULL;

	
	/* *************************************** */
	/* *     UNREGISTER THE SYSTRAY CLASS    * */
	/* *************************************** */
	
	UnregisterClass( WC_SYSTRAY, this->hInstance );

	
	/* *************************************** */
	/* *   DESTROY NECESSARY CHILD CLASSES   * */
	/* *************************************** */
	
	if ( this->settings )
		delete this->settings;
    this->settings = NULL;

	if ( this->icons )
		delete this->icons;
    this->icons = NULL;

	if ( this->shellTray )
		delete this->shellTray;
	this->shellTray = NULL;

	if ( this->szLSPath )
		delete[] this->szLSPath;

    return;

}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: LRESULT WndProc()                             * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HWND hWnd                                             * */
/* *   a handle to a window which this function handles      * */
/* *                                                         * */
/* *   UINT nMessage                                         * */
/* *   the message sent to the window                        * */
/* *                                                         * */
/* *   WPARAM wParam                                         * */
/* *   the WPARAM of this message                            * */
/* *                                                         * */
/* *   LPARAM lParam                                         * */
/* *   the LPARAM of this message                            * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: forward the window's messages                  * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

LRESULT CALLBACK grdSysTray::WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {	

	grdSysTray *theSysTray = (grdSysTray *)::GetWindowLong( hwnd, 0 );
	
	if ( theSysTray )
		return theSysTray->WindowProc(hwnd, message, wParam, lParam);
	else 
		return DefWindowProc(hwnd, message, wParam, lParam);

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: LRESULT WindowProc()                          * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HWND hWnd                                             * */
/* *   a handle to a window which this function handles      * */
/* *                                                         * */
/* *   UINT nMessage                                         * */
/* *   the message sent to the window                        * */
/* *                                                         * */
/* *   WPARAM wParam                                         * */
/* *   the WPARAM of this message                            * */
/* *                                                         * */
/* *   LPARAM lParam                                         * */
/* *   the LPARAM of this message                            * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: handle the window's messages                   * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

LRESULT grdSysTray::WindowProc( HWND hWnd, UINT nMessage, WPARAM wParam, LPARAM lParam ) {

	switch( nMessage ) {

        case LM_RESTORESYSTRAY:
		{

			this->icons->load( (void *)lParam );
			return 0;

        }
		
		case LM_SAVESYSTRAY:
		{

			this->icons->save( (void *)lParam );
			return 0;

		}
        
		case LM_SENDSYSTRAY:
		{
            /*
            // a program is requesting the system tray icons
            // such as TraySaver
			
			PSYSTRAYICON pIcon = pIconList;
			NOTIFYICONDATA nid;
			COPYDATASTRUCT cds;
			int nCount = 0;
			
			cds.dwData = wParam;
			cds.cbData = sizeof(NOTIFYICONDATA);
			
			while( pIcon ) {

				nid.cbSize = sizeof(NOTIFYICONDATA);
				nid.uID = pIcon->uID;
				nid.uFlags = pIcon->uFlags;
				nid.uCallbackMessage = pIcon->uCallbackMessage;
				nid.hIcon = pIcon->hIcon;
				nid.szTip[0] = 0;
				
				if( nid.uFlags & NIF_TIP )
					lstrcpyn( nid.szTip, pIcon->szTip, 64 );
				
				cds.lpData = (LPVOID) &nid;
				SendMessage( (HWND) lParam, WM_COPYDATA, (WPARAM) hWnd, (LPARAM) &cds );
				
				pIcon = pIcon->pNext;
				nCount++;

			}
			
			return nCount;
			
			*/
			return 0;

		}

		// this messages concerns the icons, adding, modifying and deleting
		case LM_SYSTRAY:
		{

			PNOTIFYICONDATA pnid = (PNOTIFYICONDATA) lParam;
			BOOL retVal = FALSE;
			
			switch( wParam ) {

				case NIM_ADD:

					retVal = ( this->icons->add( pnid ) != NULL );

					InvalidateRect( hWnd, NULL, TRUE );

					return retVal;

				case NIM_MODIFY:

					retVal = ( this->icons->modify( pnid ) != NULL );

					InvalidateRect( hWnd, NULL, TRUE );

					return retVal;
				
				case NIM_DELETE:
					
					retVal = this->icons->del( pnid );

					InvalidateRect( hWnd, NULL, TRUE );

					return retVal;

			}
			
			return FALSE;

		}
		
		case WM_CREATE:
		{

			// Win98/IE4 stuff
			UINT nTaskbarMessage;
			
			nTaskbarMessage = RegisterWindowMessage( TEXT("TaskbarCreated") );
			PostMessage( HWND_BROADCAST, nTaskbarMessage, 0, 0 );

			return 0;
		}
		
        // all mouseevents should be sent to the proper "icon"
		case WM_LBUTTONDBLCLK:
		case WM_LBUTTONDOWN:
		case WM_LBUTTONUP:
		case WM_MBUTTONDBLCLK:
		case WM_MBUTTONDOWN:
		case WM_MBUTTONUP:
		case WM_MOUSEMOVE:
		case WM_RBUTTONDBLCLK:
		case WM_RBUTTONDOWN:
		case WM_RBUTTONUP:
		{

			PGRDSYSTRAYICON pSysTrayIcon;
			POINT pt;
			MSG msg;

			// get cursor point
			pt.x = LOWORD(lParam);
			pt.y = HIWORD(lParam);
			
			// send message to the tooltip
			msg.hwnd = hWnd;
			msg.message = nMessage;
			msg.wParam = wParam;
			msg.lParam = lParam;
			msg.time = GetTickCount();
			msg.pt = pt;
			
			SendMessage( this->icons->hToolTip, TTM_RELAYEVENT, 0, (LPARAM) &msg );

			// get icon at point
            pSysTrayIcon = this->icons->findPt(pt);

			// post message to that icon
            if( pSysTrayIcon && (pSysTrayIcon->uFlags & NIF_MESSAGE) ) {
				PostMessage( pSysTrayIcon->hWnd, pSysTrayIcon->uCallbackMessage, (WPARAM) pSysTrayIcon->uID, (LPARAM) nMessage );
                return 0;
            }
			
			// if it's not for an icon, and it's a right click, send to ls
			if( nMessage == WM_RBUTTONUP )
				PostMessage( this->hLiteStep, LM_POPUP, (WPARAM) pt.x, (LPARAM) pt.y );
		
			return 0;
		}
		
		case WM_PAINT:
		{

            HDC     hdcBuffer;
            HBITMAP hbmBuffer, hbmBufOld;
            HRGN    hRegion;

			PAINTSTRUCT ps;
			HDC	hdcScreen;
			
			hdcScreen = BeginPaint( hWnd, &ps );

            hdcBuffer = CreateCompatibleDC(hdcScreen);
            hbmBuffer = CreateCompatibleBitmap(hdcScreen,this->settings->nTrayWidth, this->settings->nTrayHeight);
            hbmBufOld = (HBITMAP) SelectObject( hdcBuffer, hbmBuffer );

			// blit the skin
			this->settings->skinOut( hdcBuffer );

			// blit the icons
			this->icons->iconsOut( hdcBuffer );

            // the finalBmp is selected into another DC in
            // BitmapToRegion, and therefore it has to be deselected from
            // this one
            SelectObject(hdcBuffer, hbmBufOld);

	        // create a HRGN from the finalBmp, which should have been painted on
	        hRegion = BitmapToRegion(hbmBuffer, RGB(255,0,255), 0x101010, 0, 0);

	        // set the windowRgn
	        SetWindowRgn( hWnd, hRegion, TRUE );

            // Select finalBmp again to enable painting...
            hbmBufOld = (HBITMAP)SelectObject( hdcBuffer, hbmBuffer );
	
	        // when the region is applied you can use a simple SRCCOPY from the buffer
	        BitBlt(hdcScreen, 0, 0, this->settings->nTrayWidth, this->settings->nTrayHeight, hdcBuffer, 0, 0, SRCCOPY);

	        // reselect the oldBitmap and kill the new one
	        SelectObject(hdcBuffer, hbmBufOld);
	        DeleteObject(hbmBuffer);

	        // remove the two memory DCs
	        DeleteDC(hdcBuffer);

			EndPaint( hWnd, &ps );
			
			return 0;
		}
		
	}
	
	return DefWindowProc( hWnd, nMessage, wParam, lParam );

}

// BANG COMMAND FUNCTIONS

void grdTrayHide( HWND hCaller, LPCTSTR pszArgs ) {

	HWND hSysTray = FindWindow( WC_SYSTRAY, NULL );
	ShowWindow( hSysTray, SW_HIDE );

}

void grdTrayShow( HWND hCaller, LPCTSTR pszArgs ) {

	HWND hSysTray = FindWindow( WC_SYSTRAY, NULL );
	ShowWindow( hSysTray, SW_SHOWNOACTIVATE );

}

void grdTrayToggle( HWND hCaller, LPCTSTR pszArgs ) {

	HWND hSysTray = FindWindow( WC_SYSTRAY, NULL );
	BOOL visible = IsWindowVisible( hSysTray );
    ShowWindow( hSysTray, ( (visible) ? SW_HIDE : SW_SHOWNOACTIVATE) );

}

void grdTrayMove( HWND hCaller, LPCTSTR pszArgs ) {

	HWND hSysTray = FindWindow( WC_SYSTRAY, NULL );
	char token1[256] = "";
	char token2[256] = "";
	char extra[256] = "";
	int x, y;

	char* tokens[2];

	tokens[0] = token1;
	tokens[1] = token2;

	LCTokenize( pszArgs, tokens, 2, extra );

	x = atoi( token1 );
    if( x < 0 )
        x += GetSystemMetrics(SM_CXSCREEN);

	y = atoi( token2 );
    if( y < 0 )
        y += GetSystemMetrics(SM_CYSCREEN);


	SetWindowPos( hSysTray, NULL, x, y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );

}