#define WIN32_LEAN_AND_MEAN
#define STRICT

#include <windows.h>
#include "grdSysTraySettings.h"
#include "lsapi.h"

int OmniBlt (HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, BOOL bTiled);


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: grdSysTraySettings()                          * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: initialize the sysTraySettings class           * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

grdSysTraySettings::grdSysTraySettings() {

	BOOL bAdjustSize = FALSE;
    char *tmpString = new char[256];

	// style booleans
    this->bAlwaysOnTop = GetRCBool("grdTrayAlwaysOnTop",TRUE);
	this->bVisible = GetRCBool("grdTrayHidden",FALSE);
	this->bHideIfEmpty = GetRCBool("grdTrayHideIfEmpty",TRUE);

	// tray
	this->nTrayX = GetRCInt("grdTrayX",0);
    if(this->nTrayX < 0)
        this->nTrayX += GetSystemMetrics(SM_CXSCREEN);

	this->nTrayY = GetRCInt("grdTrayY",0);
    if(this->nTrayY < 0)
        this->nTrayY += GetSystemMetrics(SM_CYSCREEN);

	this->nTrayWidth = GetRCInt("grdTrayWidth",100);
	this->nTrayHeight = GetRCInt("grdTrayHeight",20);

	// border
	this->uBorderTop = GetRCInt("grdTrayBorderTop",0);
	this->uBorderLeft = GetRCInt("grdTrayBorderLeft",0);
	this->uBorderRight = GetRCInt("grdTrayBorderRight",0);
	this->uBorderBottom = GetRCInt("grdTrayBorderBottom",0);
	
	// skinning
    GetRCString("grdTrayBitmap",tmpString,"",256);

    if ( tmpString[0] == '\0' )
        this->hbmSkin = NULL;
    else
        this->hbmSkin = LoadLSImage(tmpString, NULL);

    this->bSkinTiled = GetRCBool("grdTrayBitmapTiled",TRUE);

	// icon effects
	this->uEffectFlags = 0;

	this->uHueIntensity = (UCHAR)GetRCInt("grdTrayIconHueIntensity",0);
	this->clrHue = GetRCColor("grdTrayIconHueColor",RGB(128,128,128));
	if ( this->uHueIntensity )
		this->uEffectFlags |= FX_ICONHUE;

	this->uSaturnation = (UCHAR)GetRCInt("grdTrayIconSaturnation", 255);
	if ( this->uSaturnation != 255 )
		this->uEffectFlags |= FX_SATURNATION;

	// colors
	this->clrBack = GetRCColor("grdTrayBGColor",RGB(255,0,255));
	this->clrBorder = GetRCColor("grdTrayBorderColor",RGB(255,255,255));

	//tray wrapping
    this->uWrapCount = GetRCInt("grdTrayWrapCount",0);

    // get direction
    GetRCString("grdTrayDirection",tmpString,"right",256);
    if ( !strnicmp(tmpString, "left", 5) )
        this->uDirection = TD_LEFT;
    else if ( !strnicmp(tmpString, "up", 3) )
        this->uDirection = TD_UP;
    else if ( !strnicmp(tmpString, "down", 5) )
        this->uDirection = TD_DOWN;
	else
		this->uDirection = TD_RIGHT;

    // get wrap-direction
    GetRCString("grdTrayWrapDirection",tmpString,"up",256);
    if (this->uDirection & TD_VERTICAL) {
        if ( !strnicmp( tmpString, "left", 5 ) )
            this->uWrapDirection = TD_LEFT;
        else 
            this->uWrapDirection = TD_RIGHT;
    } else {
        if ( !strnicmp( tmpString, "down", 5) )
            this->uWrapDirection = TD_DOWN;
        else
            this->uWrapDirection = TD_UP;
    }

	// icon
	this->uIconSize = GetRCInt("grdTrayIconSize",16);
    this->nIconSpacingX = GetRCInt("grdTrayIconSpacingX",1);
	this->nIconSpacingY = GetRCInt("grdTrayIconSpacingY",1);

    // this section evaluates how the placing of new icons should be calculated
    this->nDeltaX = ((( this->uDirection & TD_VERTICAL )?this->uWrapDirection:this->uDirection ) == TD_LEFT) ? -1 : 1;
    this->nDeltaY = ((( this->uDirection & TD_VERTICAL )?this->uDirection:this->uWrapDirection ) == TD_DOWN) ? 1 : -1;

	this->nDeltaX *= this->uIconSize + this->nIconSpacingX;
	this->nDeltaY *= this->uIconSize + this->nIconSpacingY;

	// the following settings are used with the icon rectangles
	// to ensure they match the desired pattern
	this->nFirstX = ( (this->uDirection == TD_LEFT) || (this->uWrapDirection == TD_LEFT) ) ?
						( this->nTrayWidth -  this->uBorderRight  - this->nIconSpacingX - this->uIconSize ) :
						( this->uBorderLeft + this->nIconSpacingX );
	this->nLastX  = this->nFirstX + ( this->uWrapCount - 1 ) * this->nDeltaX;

	this->nFirstY = ( (this->uDirection == TD_DOWN) || (this->uWrapDirection == TD_DOWN) ) ?
						( this->uBorderTop + this->nIconSpacingY ) :
						( this->nTrayHeight - this->uBorderBottom - this->nIconSpacingY - this->uIconSize );

	this->nLastY  = this->nFirstY + ( this->uWrapCount - 1 ) * this->nDeltaY;
	
	delete[] tmpString;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: ~grdSysTraySettings()                         * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: destroy the sysTraySettings class              * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

grdSysTraySettings::~grdSysTraySettings() {

	if ( this->hbmSkin )
		DeleteObject( this->hbmSkin );

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: void skinOut()                                * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HDC hdcDest                                           * */
/* *   the destination dc                                    * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: put the layout on a specific dc                * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

void grdSysTraySettings::skinOut( HDC hdcDest ) {

	RECT r;
	HBRUSH brush;

    /* ************************************** */
    /* * fill the standard backgroundcolors * */
    /* ************************************** */

	r.left   = 0;
	r.top    = 0;
	r.right  = this->nTrayWidth;
	r.bottom = this->nTrayHeight;

	brush = CreateSolidBrush( this->clrBorder );

    // fill complete background with border color
    FillRect(hdcDest, &r, brush );

	DeleteObject( brush );

	r.left   = this->uBorderLeft;
	r.top    = this->uBorderTop;
	r.right  = this->nTrayWidth  - this->uBorderRight;
	r.bottom = this->nTrayHeight - this->uBorderBottom;
    
	brush = CreateSolidBrush( this->clrBack );

	// now fill the inner rectangle (excluding the borders) with the background color
    FillRect(hdcDest, &r, brush );

	DeleteObject( brush );

    /* ************************************** */
    /* * bitblt the image parts there       * */
    /* * oops, large block of code here     * */
    /* ************************************** */

    if ( this->hbmSkin != NULL ) {

        HDC     hdcSrc;
        HBITMAP oldSrc;
        BITMAP  bm;

        hdcSrc = CreateCompatibleDC( hdcDest );
        oldSrc = (HBITMAP) SelectObject( hdcSrc, this->hbmSkin );
        GetObject( this->hbmSkin, sizeof(BITMAP), &bm );

        // take the four corners to start with
        // topleft
	    TransparentBltLS( hdcDest,								// dst
                          0,                 0,					// -pos
                          this->uBorderLeft, this->uBorderTop,	// size
                          hdcSrc,								// src	
                          0,                 0,					// -pos
                          RGB(255,0,255));

        // topright
        TransparentBltLS( hdcDest,								// dst
                          this->nTrayWidth-this->uBorderRight, 0,	// -pos
                          this->uBorderRight, this->uBorderTop, // size
                          hdcSrc,								// src
                          bm.bmWidth-this->uBorderRight,   0,   // -pos
                          RGB(255,0,255));
        //bottomright
        TransparentBltLS( hdcDest,								// dst
					      this->nTrayWidth-this->uBorderRight, this->nTrayHeight-this->uBorderBottom,	// -pos
                          this->uBorderRight, this->uBorderBottom, // size
                          hdcSrc,								// src
					      bm.bmWidth-this->uBorderRight, bm.bmHeight-this->uBorderBottom,	// -pos
                          RGB(255,0,255));
        //bottomleft
        TransparentBltLS( hdcDest,
                          0,                 this->nTrayHeight-this->uBorderBottom,	// -pos
                          this->uBorderLeft, this->uBorderBottom,	// size
                          hdcSrc,								// src	
                          0,                 bm.bmHeight-this->uBorderBottom,	// -pos
                          RGB(255,0,255));

        // top
        OmniBlt( hdcDest, // dst
                 this->uBorderLeft, 0, // -pos
                 this->nTrayWidth-(this->uBorderRight+this->uBorderLeft), this->uBorderTop, // -size
                 hdcSrc, // src
                 this->uBorderLeft, 0, // -pos
                 bm.bmWidth-(this->uBorderRight+this->uBorderLeft), this->uBorderTop, // -size
                 this->bSkinTiled);

        // right
        OmniBlt( hdcDest, // dst
                 this->nTrayWidth-this->uBorderRight, this->uBorderTop, // -pos
                 this->uBorderRight, this->nTrayHeight-(this->uBorderBottom+this->uBorderTop), // -size
                 hdcSrc, // src
                 bm.bmWidth-this->uBorderRight, this->uBorderTop, // -pos
                 this->uBorderRight, bm.bmHeight-(this->uBorderBottom+this->uBorderTop), // -size
                 this->bSkinTiled);

        // bottom
        OmniBlt( hdcDest, // dst
                 this->uBorderLeft, this->nTrayHeight-this->uBorderBottom, //-pos
                 this->nTrayWidth-(this->uBorderRight+this->uBorderLeft), this->uBorderBottom, // -size
                 hdcSrc, // src
                 this->uBorderLeft, bm.bmHeight-this->uBorderBottom, //-pos
                 bm.bmWidth-(this->uBorderRight+this->uBorderLeft), this->uBorderBottom, // -size
                 this->bSkinTiled);

        // left
        OmniBlt( hdcDest, // dst
                 0, this->uBorderTop, //-pos
                 this->uBorderLeft, this->nTrayHeight-(this->uBorderBottom+this->uBorderTop), // -size
                 hdcSrc, // src
                 0, this->uBorderTop, //-pos
                 this->uBorderLeft, bm.bmHeight-(this->uBorderBottom+this->uBorderTop), // -size
                 this->bSkinTiled);

        // middle area
        OmniBlt( hdcDest,
                 this->uBorderLeft, this->uBorderTop, //-pos
                 this->nTrayWidth-(this->uBorderRight+this->uBorderLeft), this->nTrayHeight-(this->uBorderBottom+this->uBorderTop), // -size
                 hdcSrc,
                 this->uBorderLeft, this->uBorderTop, //-pos
                 bm.bmWidth-(this->uBorderRight+this->uBorderLeft), bm.bmHeight-(this->uBorderBottom+this->uBorderTop), // -size
                 this->bSkinTiled);

        SelectObject(hdcSrc, oldSrc);
        DeleteDC( hdcSrc );

    }
	return;

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * FUNCTION: int OmniBlt()                                 * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PARAMETERS:                                             * */
/* *                                                         * */
/* *   HDC hdcDest                                           * */
/* *   the destination dc                                    * */
/* *                                                         * */
/* *   int xDest                                             * */
/* *   the destination x                                     * */
/* *                                                         * */
/* *   int yDest                                             * */
/* *   the destination y                                     * */
/* *                                                         * */
/* *   int cxDest                                            * */
/* *   the destination width                                 * */
/* *                                                         * */
/* *   int cyDest                                            * */
/* *   the destination height                                * */
/* *                                                         * */
/* *   HDC hdcSrc                                            * */
/* *   the source dc                                         * */
/* *                                                         * */
/* *   int xSrc                                              * */
/* *   the source x                                          * */
/* *                                                         * */
/* *   int ySrc                                              * */
/* *   the source y                                          * */
/* *                                                         * */
/* *   int cxSrc                                             * */
/* *   the source width                                      * */
/* *                                                         * */
/* *   int cySrc                                             * */
/* *   the source height                                     * */
/* *                                                         * */
/* *   BOOL bTiled                                           * */
/* *   tile or stretch                                       * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* *                                                         * */
/* * PURPOSE: tile or stretch one dc on another              * */
/* *                                                         * */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

int OmniBlt (HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, BOOL bTiled) {

	if( bTiled ) {

		int nWidth = cxDest;
		int nHeight = cyDest;

		int x, y;
		
		for( y = 0; y < nHeight; y += cySrc ) {

			cxDest = nWidth;
			
			for( x = 0; x < nWidth; x += cxSrc ) {

				TransparentBltLS( hdcDest, xDest + x, yDest + y, min( cxDest, cxSrc ), min( cyDest, cySrc ), hdcSrc, xSrc, ySrc, RGB(255,0,255) );
				cxDest -= cxSrc;

			}
			
			cyDest -= cySrc;

		}
		return 0;

	} else
		return StretchBlt( hdcDest, xDest, yDest, cxDest, cyDest, hdcSrc, xSrc, ySrc, cxSrc, cySrc, SRCCOPY );

}

